package at.tuwien;

import org.jboss.logging.Logger;
import org.keycloak.events.Event;
import org.keycloak.events.EventListenerProvider;
import org.keycloak.events.EventType;
import org.keycloak.events.admin.AdminEvent;
import org.keycloak.events.admin.OperationType;
import org.keycloak.events.admin.ResourceType;
import org.keycloak.models.KeycloakSession;
import org.keycloak.models.RealmModel;
import org.keycloak.models.RealmProvider;
import org.keycloak.models.UserModel;

import java.util.StringJoiner;

public class CreateEventListenerProvider implements EventListenerProvider {

    private static final Logger log = Logger.getLogger(CreateEventListenerProvider.class);

    private final KeycloakSession session;
    private final RealmProvider model;

    public CreateEventListenerProvider(KeycloakSession session) {
        this.session = session;
        this.model = session.realms();
    }

    @Override
    public void onEvent(Event event) {
        if (EventType.REGISTER.equals(event.getType()) || EventType.IDENTITY_PROVIDER_FIRST_LOGIN.equals(event.getType())) {
            event.getDetails().forEach((key, value) -> log.debugf("%s : %s", key, value));
            RealmModel realm = this.model.getRealm(event.getRealmId());
            UserModel user = this.session.users().getUserById(realm, event.getUserId());
            sendUserData(user);
        }

    }

    @Override
    public void onEvent(AdminEvent adminEvent, boolean b) {
        log.debug("onEvent(AdminEvent)");
        log.debugf("Resource path: %s", adminEvent.getResourcePath());
        log.debugf("Resource type: %s", adminEvent.getResourceType());
        log.debugf("Operation type: %s", adminEvent.getOperationType());
        log.debugf("AdminEvent.toString(): %s", toString(adminEvent));
        if (ResourceType.USER.equals(adminEvent.getResourceType())
                && OperationType.CREATE.equals(adminEvent.getOperationType())) {
            RealmModel realm = this.model.getRealm(adminEvent.getRealmId());
            UserModel user = this.session.users().getUserById(realm, adminEvent.getResourcePath().substring(6));

            sendUserData(user);
        }
    }

    private void sendUserData(UserModel user) {
        final String userData = "{" +
                quoteAttr("id", user.getId()) + ", " +
                quoteAttr("username", user.getUsername()) + ", " +
                quoteAttr("ldap_id", user.getFirstAttribute("LDAP_ID")) + ", " +
                quoteAttr("given_name", user.getFirstName()) + ", " +
                quoteAttr("family_name", user.getLastName()) +
                "}";
        try {
            log.debugf("create new user in API: %s", userData);
            Client.postService(userData);
        } catch (Exception e) {
            log.errorf("Failed to call API: %s", e);
        }
    }

    private static String quoteAttr(String key, String value) {
        if (value == null || value.isBlank() || value.isEmpty() || value.contentEquals(" ")) {
            return "\"" + key + "\": null";
        }
        return "\"" + key + "\": \"" + value + "\"";
    }

    @Override
    public void close() {
    }

    private String toString(Event event) {
        final StringJoiner joiner = new StringJoiner(", ");
        joiner.add("type=" + event.getType())
                .add("realmId=" + event.getRealmId())
                .add("clientId=" + event.getClientId())
                .add("userId=" + event.getUserId())
                .add("ipAddress=" + event.getIpAddress());
        if (event.getError() != null) {
            joiner.add("error=" + event.getError());
        }
        if (event.getDetails() != null) {
            event.getDetails().forEach((key, value) -> {
                if (value == null || !value.contains(" ")) {
                    joiner.add(key + "=" + value);
                } else {
                    joiner.add(key + "='" + value + "'");
                }
            });
        }
        return joiner.toString();
    }

    private String toString(AdminEvent event) {
        RealmModel realm = this.model.getRealm(event.getRealmId());
        UserModel newRegisteredUser = this.session.users().getUserById(realm, event.getAuthDetails().getUserId());

        StringJoiner joiner = new StringJoiner(", ");

        joiner.add("operationType=" + event.getOperationType())
                .add("realmId=" + event.getAuthDetails().getRealmId())
                .add("clientId=" + event.getAuthDetails().getClientId())
                .add("userId=" + event.getAuthDetails().getUserId());

        if (newRegisteredUser != null) {
            joiner.add("email=" + newRegisteredUser.getEmail())
                    .add("username=" + newRegisteredUser.getUsername())
                    .add("firstName=" + newRegisteredUser.getFirstName())
                    .add("lastName=" + newRegisteredUser.getLastName());
        }

        joiner.add("ipAddress=" + event.getAuthDetails().getIpAddress())
                .add("resourcePath=" + event.getResourcePath());

        if (event.getError() != null) {
            joiner.add("error=" + event.getError());
        }

        return joiner.toString();
    }
}
